--
-- StopBuyStuff
--
-- @author  ANDR55
-- Version 0.1
-- @date 23.10.2013


StopBuyStuff = {};


function SowingMachine:updateTick(dt)
    self.isSowingSpeedLimitActive = false;
    if self:getIsActive() then
    
        self.lastSowingArea = 0;
        local showFieldNotOwnedWarning = false;
        if self.isServer then
            local hasGroundContact = self:getIsTypedWorkAreaActive(WorkArea.AREATYPE_SOWINGMACHINE);
            
            if self.sowingMachineHasGroundContact ~= hasGroundContact then
                self:raiseDirtyFlags(self.sowingMachineGroundContactFlag);

                self.sowingMachineHasGroundContact = hasGroundContact;
            end;
        end;
        local hasGroundContact = self.sowingMachineHasGroundContact;

        local doGroundManipulation = (self.movingDirection > 0 and hasGroundContact and (not self.needsActivation or self:getIsTurnedOn()));

        local foldAnimTime = self.foldAnimTime;
        if doGroundManipulation then
            self.isSowingSpeedLimitActive = true;
            if self.isServer then
                local hasSeeds = (self.fillLevel > 0);
                local useFillLevel = true;
                if self:getCapacity() == 0 or self:getIsHired() then
                    useFillLevel = hasSeeds;
                    hasSeeds = true;
                end;
                if hasSeeds then
                    local workAreasSend, showWarning, _ = self:getTypedNetworkAreas(WorkArea.AREATYPE_SOWINGMACHINE, true);                        
                    showFieldNotOwnedWarning = showWarning;

                    if (table.getn(workAreasSend) > 0) then
                        local seedsFruitType = self.seeds[self.currentSeed];
                        local dx,dy,dz = localDirectionToWorld(self.sowingDirectionNode, 0, 0, 1);

                        local angleRad = Utils.getYRotationFromDirection(dx, dz)
                        local desc = FruitUtil.fruitIndexToDesc[seedsFruitType];
                        if desc ~= nil and desc.directionSnapAngle ~= 0 then
                            angleRad = math.floor(angleRad / desc.directionSnapAngle + 0.5) * desc.directionSnapAngle;
                        end

                        local angle = Utils.convertToDensityMapAngle(angleRad, g_currentMission.terrainDetailAngleMaxValue);

                        local area, detailArea = SowingMachineAreaEvent.runLocally(workAreasSend, seedsFruitType, angle, self.useDirectPlanting)
                        if area > 0 or detailArea > 0 then
                            if area > 0 then
                                local fruitDesc = FruitUtil.fruitIndexToDesc[seedsFruitType];
                                self.lastSowingArea = Utils.areaToHa(area, g_currentMission:getFruitPixelsToSqm());
                                local usage = fruitDesc.seedUsagePerSqm * self.lastSowingArea * 10000;                                    
                                
                                g_currentMission.missionStats:updateStats("seedUsage", usage);
                                g_currentMission.missionStats:updateStats("hectaresSeeded", self.lastSowingArea);

                                if useFillLevel then
                                    self:setFillLevel(self.fillLevel - usage, self.currentFillType);
                                else
                                    local fillTypeDesc = Fillable.fillTypeIndexToDesc[Fillable.FILLTYPE_SEEDS]
                                    if fillTypeDesc ~= nil then
                                        local price = usage*fillTypeDesc.pricePerLiter*1.5;                 -- increase price if AI is active to reward the player's manual work
                                        g_currentMission.missionStats:updateStats("expenses", price);
                                        g_currentMission:addSharedMoney(-price, "other");
                                    end
                                end
                            end
                            g_server:broadcastEvent(SowingMachineAreaEvent:new(workAreasSend, seedsFruitType, angle, self.useDirectPlanting));
                        end;
                    end;
                end;
            end;
            g_currentMission.missionStats:updateStats("seedingDuration", dt/(1000*60));
        end;
        
        if self.isClient then
            if doGroundManipulation and self:getLastSpeed() > 3 then
                if self:getIsActiveForSound() then
                    Utils.playSample(self.sampleSowing, 0, 0, nil);
                end
            else
                Utils.stopSample(self.sampleSowing);
            end

            if self:getIsTurnedOn() and self:getIsActiveForSound() then
                Utils.playSample(self.sampleAirBlower, 0, 0, nil);
            end
        end
        
        if self.isServer then
            if showFieldNotOwnedWarning ~= self.showFieldNotOwnedWarning then
                self.showFieldNotOwnedWarning = showFieldNotOwnedWarning;
                self:raiseDirtyFlags(self.sowingMachineGroundContactFlag);
            end
        end
    end;
    
    if self.isClient then 
        if self.lastFillLevel ~= self.fillLevel and self.fillLevel > 0 then
            if self.lastFillLevel < self.fillLevel and self.lastFillLevel ~= -1 then
                self.sampleFillStopTime = g_currentMission.time + 150;
            end;
            self.lastFillLevel = self.fillLevel;
        end;
        
        if self.sampleFillStopTime > g_currentMission.time then
            if self:getIsActiveForSound(true) then
                Utils.playSample(self.sampleFill, 0, 0, nil);
                Utils.stop3DSample(self.sampleFill);
            else
                Utils.stopSample(self.sampleFill);
                Utils.play3DSample(self.sampleFill);
            end;
        else
            Utils.stopSample(self.sampleFill);
            Utils.stop3DSample(self.sampleFill);
        end;
    end;
end;


function Sprayer:updateTick(dt)
    self.isSprayerSpeedLimitActive = false;
    if self:getIsActive() then
        local showFieldNotOwnedWarning = false;
        if self.turnOnDueToLoweredImplement ~= nil then
            if self:getIsTurnedOn() == false then
                self:setIsTurnedOn(true);
            end
        end
        if self:getIsTurnedOn() then
            if not self:getIsTurnedOnAllowed(true) then
                self:setIsTurnedOn(false);
            end
        end
        if self:getIsTurnedOn() and self.allowsSpraying then
            self.isSprayerSpeedLimitActive = true;
            local readyToSpray = self:getIsReadyToSpray();
            local fillType = self.currentFillType;
            if fillType == Fillable.FILLTYPE_UNKNOWN then
                fillType = self:getFirstEnabledFillType();
            end

            if self.isClient then
                -- update valve particle systems
                local foldAnimTime;
                if self.doorAnimation ~= nil and self.doorAnimation.name ~= nil then
                    foldAnimTime = self:getAnimationTime(self.doorAnimation.name);
                end
                if (foldAnimTime ~= nil and foldAnimTime ~= self.lastSprayValveUpdateFoldTime) or self.lastTurnedOn == false then

                    self.lastSprayValveUpdateFoldTime = foldAnimTime;
                    self.lastTurnedOn = true;

                    if self:getAreEffectsVisible() then
                        if foldAnimTime ~= nil then
                            for _,sprayValve in pairs(self.sprayValves) do
                                readyToSpray = readyToSpray and (foldAnimTime > sprayValve.foldMinLimit);
                                Utils.setEmittingState(sprayValve.particleSystems, foldAnimTime <= sprayValve.foldMaxLimit and foldAnimTime >= sprayValve.foldMinLimit);
                            end

                            if self.sprayParticleSystems[fillType] ~= nil then
                                local ps = self.sprayParticleSystems[fillType];
                                readyToSpray = readyToSpray and (foldAnimTime > ps.foldMinLimit);
                                if foldAnimTime <= ps.foldMaxLimit and foldAnimTime >= ps.foldMinLimit then
                                    Utils.setEmittingState(ps, true);
                                else
                                    Utils.setEmittingState(ps, false);
                                end
                            end

                            for _, effect in pairs(self.sprayerEffects) do
                                readyToSpray = readyToSpray and (foldAnimTime > effect.foldMinLimit);
                                if foldAnimTime <= effect.foldMaxLimit and foldAnimTime >= effect.foldMinLimit then
                                    effect:setFillType(fillType);
                                    EffectManager:startEffect(effect);
                                else
                                    EffectManager:stopEffect(effect);
                                end
                            end
                        else
                            for _,sprayValve in pairs(self.sprayValves) do
                                Utils.setEmittingState(sprayValve.particleSystems, true);
                            end
                            if self.sprayParticleSystems[fillType] ~= nil then
                                Utils.setEmittingState(self.sprayParticleSystems[fillType], true);
                            end
                            for _, effect in pairs(self.sprayerEffects) do
                                effect:setFillType(fillType);
                                EffectManager:startEffect(effect);
                            end
                        end
                    end
                end
            end

            self.lastSprayingArea = 0;

            if self.isServer and readyToSpray then
                local litersPerSecond = self:getLitersPerSecond(fillType);
                local usage = litersPerSecond * dt*0.001;
                g_currentMission.missionStats:updateStats("sprayUsage", usage);

                local hasSpray = false;

                --[[
                if self:getCapacity() == 0 or self:getIsHired() then
                    hasSpray = true;
                    local fillTypeDesc = Fillable.fillTypeIndexToDesc[fillType];
                    if fillTypeDesc ~= nil then
                        local delta = usage*fillTypeDesc.pricePerLiter;
                        g_currentMission.missionStats:updateStats("expenses", delta);
                        g_currentMission:addSharedMoney(-delta, "other");
                    end
                else
                ]]--
                if self.fillLevel > 0 then
                    hasSpray = true;

                    local info = self.fillVolumeDischargeInfo;

                    if usage > 0 then
                        if info.alsoUseLoadInfoForDischarge and self.fillVolumeLoadInfo ~= nil then
                            local fillDeltaLoadInfo = usage*info.loadInfoFillFactor;
                            usage = usage - fillDeltaLoadInfo;
                            local x,y,z = getWorldTranslation(self.fillVolumeLoadInfo.node);
                            local d1x,d1y,d1z = localDirectionToWorld(self.fillVolumeLoadInfo.node, self.fillVolumeLoadInfo.width*info.loadInfoSizeScale[1],0,0);
                            local d2x,d2y,d2z = localDirectionToWorld(self.fillVolumeLoadInfo.node, 0,0,self.fillVolumeLoadInfo.length*info.loadInfoSizeScale[2]);
                            local fss = {x=x,y=y,z=z, d1x=d1x,d1y=d1y,d1z=d1z, d2x=d2x,d2y=d2y,d2z=d2z};
                            self:setFillLevel(self.fillLevel - fillDeltaLoadInfo, fillType, false, fss);
                        end
                    end

                    local x,y,z = getWorldTranslation(info.node);
                    local d1x,d1y,d1z = localDirectionToWorld(info.node, info.width,0,0);
                    local d2x,d2y,d2z = localDirectionToWorld(info.node, 0,0,info.length);
                    local fillSourceStruct = {x=x,y=y,z=z, d1x=d1x,d1y=d1y,d1z=d1z, d2x=d2x,d2y=d2y,d2z=d2z};

                    self:setFillLevel(self.fillLevel - usage, fillType, false, fillSourceStruct);
                else
                    -- try to find another attached sprayer
                    local sprayerTank = Sprayer.findAttachedSprayerTank(self:getRootAttacherVehicle(), fillType, self.needsTankActivation);
                    if sprayerTank ~= nil then
                        hasSpray = sprayerTank:getFillLevel(fillType) > 0;

                        if hasSpray then
                            local x,y,z = getWorldTranslation(sprayerTank.fillVolumeDischargeInfo.node);
                            local d1x,d1y,d1z = localDirectionToWorld(sprayerTank.fillVolumeDischargeInfo.node, sprayerTank.fillVolumeDischargeInfo.width,0,0);
                            local d2x,d2y,d2z = localDirectionToWorld(sprayerTank.fillVolumeDischargeInfo.node, 0,0,sprayerTank.fillVolumeDischargeInfo.length);
                            local fillSourceStruct = {x=x,y=y,z=z, d1x=d1x,d1y=d1y,d1z=d1z, d2x=d2x,d2y=d2y,d2z=d2z};

                            sprayerTank:setFillLevel(sprayerTank:getFillLevel(fillType) - usage, fillType, false, fillSourceStruct);
                        end
                    end
                end

                if not hasSpray and (self:getCapacity() == 0 or self:getIsHired()) then
                    hasSpray = true;
                    local fillTypeDesc = Fillable.fillTypeIndexToDesc[fillType];
                    if fillTypeDesc ~= nil then
                        local delta = usage*fillTypeDesc.pricePerLiter*1.5;             -- increase price if AI is active to reward the player's manual work
                        g_currentMission.missionStats:updateStats("expenses", delta);
                        g_currentMission:addSharedMoney(-delta, "other");
                    end
                end

                if hasSpray then
                    local workAreasSend, showWarning, area = self:getTypedNetworkAreas(WorkArea.AREATYPE_SPRAYER, true);
                    showFieldNotOwnedWarning = showWarning;
                    self.lastSprayingArea = area;

                    if (table.getn(workAreasSend) > 0) then
                        SprayerAreaEvent.runLocally(workAreasSend);
                        g_server:broadcastEvent(SprayerAreaEvent:new(workAreasSend));
                    end
                end
            end

            if self.isClient and self:getIsActiveForSound() then
                Utils.playSample(self.sampleSprayer, 0, 0, nil);
            end
        else
            if self.isClient then
                for _,sprayValve in pairs(self.sprayValves) do
                    Utils.setEmittingState(sprayValve.particleSystems, false);
                end
                for _, particleSystem in pairs(self.sprayParticleSystems) do
                    Utils.setEmittingState(particleSystem, false);
                end
                for _, effect in pairs(self.sprayerEffects) do
                    EffectManager:stopEffect(effect);
                end
            end
        end
        self.lastTurnedOn = self:getIsTurnedOn();

        if self.isServer then
            if showFieldNotOwnedWarning ~= self.showFieldNotOwnedWarning then
                self.showFieldNotOwnedWarning = showFieldNotOwnedWarning;
                self:raiseDirtyFlags(self.sprayerDirtyFlag);
            end
        end
    end

    if self.isClient then
        if self.lastFillLevel ~= self.fillLevel and self.fillLevel > 0 then
            if self.lastFillLevel < self.fillLevel and self.lastFillLevel ~= -1 then
                self.sampleFillStopTime = g_currentMission.time + 150;
            end
            self.lastFillLevel = self.fillLevel;
        end

        if self.sampleFillStopTime > g_currentMission.time then
            if self:getIsActiveForSound(true) then
                Utils.playSample(self.sampleFill, 0, 0, nil);
                Utils.stop3DSample(self.sampleFill);
            else
                Utils.stopSample(self.sampleFill);
                Utils.play3DSample(self.sampleFill);
            end
        else
            Utils.stopSample(self.sampleFill);
            Utils.stop3DSample(self.sampleFill);
        end
    end
end

